from thinc.api import Model, chain
from .split_trf import split_trf_batch


def replace_listener(model):
    return chain(model, split_trf_batch())


def replace_listener_cfg(tok2vec_model_cfg, listener_model_cfg):
    result = tok2vec_model_cfg
    if (
        "TransformerModel" in tok2vec_model_cfg["@architectures"]
        and "TransformerListener" in listener_model_cfg["@architectures"]
    ):
        result["@architectures"] = "spacy-transformers.Tok2VecTransformer.v1"
        if "pooling" in listener_model_cfg and "pooling" not in result:
            result["pooling"] = listener_model_cfg["pooling"]
        if "grad_factor" in listener_model_cfg and "grad_factor" not in result:
            result["grad_factor"] = listener_model_cfg["pooling"]

    return result



from spacy import Language

from spacy_transformers import TransformerData
import srsly


def test_serialize_transformer_data():
    data = {"x": TransformerData.empty()}
    bytes_data = srsly.msgpack_dumps(data)
    new_data = srsly.msgpack_loads(bytes_data)
    assert isinstance(new_data["x"], TransformerData)


def test_transformer_tobytes():
    nlp = Language()
    trf = nlp.add_pipe("transformer")
    trf_bytes = trf.to_bytes()

    nlp2 = Language()
    trf2 = nlp2.add_pipe("transformer")
    trf2.from_bytes(trf_bytes)


def test_transformer_model_tobytes():
    nlp = Language()
    trf = nlp.add_pipe("transformer")
    nlp.initialize()
    trf_bytes = trf.to_bytes()

    nlp2 = Language()
    trf2 = nlp2.add_pipe("transformer")
    nlp2.initialize()
    trf2.from_bytes(trf_bytes)
